#!/usr/bin/env python

import sys
from threading import Thread

import obd
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QApplication

from honda_dashboard.components import DashboardWindow, DashboardWidget, InitialScreen
from honda_dashboard.config import Config
from honda_dashboard.indicators import IndicatorBox, Limits
from honda_dashboard.toolbar import Toolbar


class Dashboard:

    def __init__(self):
        self.__config = Config()
        self.__app = QApplication([])
        self.__main_window = DashboardWindow(self.__config)
        self.__toolbar = Toolbar(self.__config)
        self.__main_widget = DashboardWidget()

        self.__build_indicators()

    def run(self):
        self.__toolbar.toggle_night_mode.triggered.connect(self.__night_mode_toggled)
        self.__main_window.addToolBar(Qt.RightToolBarArea, self.__toolbar)
        self.__main_window.show()

        initial_screen = InitialScreen()
        self.__main_window.setCentralWidget(initial_screen)
        initial_screen.show()

        conn_thread = Thread(target=self.__connection_init)
        conn_thread.start()

        sys.exit(self.__app.exec())

    def __build_indicators(self):
        self.__speed = IndicatorBox('Speed', 'km/h', self.__config, True, Limits(50, 90, 140))
        self.__main_widget.layout().addWidget(self.__speed, 0, 0)

        self.__rpm = IndicatorBox('RPM', 'rpm', self.__config, True, Limits(2500, 3500, 5500))
        self.__main_widget.layout().addWidget(self.__rpm, 0, 1)

        self.__coolant_temp = IndicatorBox('Coolant', '°C', self.__config, True, Limits(45, 85, 100))
        self.__main_widget.layout().addWidget(self.__coolant_temp, 0, 2)

        self.__engine_load = IndicatorBox('Engine', '%', self.__config, True, Limits(40, 60, 80))
        self.__main_widget.layout().addWidget(self.__engine_load, 1, 0)

        self.__intake_temp = IndicatorBox('Intake', '°C', self.__config, True)
        self.__main_widget.layout().addWidget(self.__intake_temp, 1, 1)

        self.__intake_press = IndicatorBox('Intake', 'kPa', self.__config, True)
        self.__main_widget.layout().addWidget(self.__intake_press, 1, 2)

    def __night_mode_toggled(self):
        self.__config.set('MODE', 'night_mode_on', '0' if self.__config.night_mode_on else '1')
        self.__main_window.handle_mode()
        self.__toolbar.handle_mode()
        self.__speed.handle_mode()
        self.__rpm.handle_mode()
        self.__coolant_temp.handle_mode()
        self.__engine_load.handle_mode()
        self.__intake_temp.handle_mode()
        self.__intake_press.handle_mode()

    def __connection_init(self):
        connection = obd.Async(self.__config.device)

        if connection.is_connected():
            self.__main_window.setCentralWidget(self.__main_widget)

        connection.watch(obd.commands.SPEED, callback=self.__speed.update_value)
        connection.watch(obd.commands.RPM, callback=self.__rpm.update_value)
        connection.watch(obd.commands.COOLANT_TEMP, callback=self.__coolant_temp.update_value)
        connection.watch(obd.commands.ENGINE_LOAD, callback=self.__engine_load.update_value)
        connection.watch(obd.commands.INTAKE_TEMP, callback=self.__intake_temp.update_value)
        connection.watch(obd.commands.INTAKE_PRESSURE, callback=self.__intake_press.update_value)
        connection.start()




import os

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import QMainWindow, QWidget, QGridLayout, QLabel, QVBoxLayout

from honda_dashboard.config import Config


class DashboardWindow(QMainWindow):

    def __init__(self, cfg: Config):
        super().__init__()

        self.__config = cfg
        self.setWindowTitle('Dashboard')
        self.handle_mode()

    def handle_mode(self):
        if self.__config.night_mode_on:
            color = self.__config.night_colors['bg']
        else:
            color = self.__config.day_colors['bg']

        self.setStyleSheet('background-color: #' + color + ';')


class DashboardWidget(QWidget):

    def __init__(self):
        super().__init__()

        self.setLayout(QGridLayout())


class InitialScreen(QWidget):

    def __init__(self):
        super().__init__()

        self.setLayout(QVBoxLayout())
        loading = QLabel('LOADING...')
        font = QFont('Arial', 32)
        font.setBold(True)
        loading.setFont(font)
        loading.setAlignment(Qt.AlignCenter)
        self.layout().addWidget(loading)
        self.setMinimumSize(480, 320)
        path = os.path.dirname(os.path.abspath(__file__))
        self.setStyleSheet('background-image: url(' + path + '/honda.png); background-position: center; '
                           'background-repeat: no-repeat;')
        loading.setStyleSheet('color: #ed323e;')




import configparser
import os


class Config:
    night_mode_on = None
    device = None
    day_colors = []
    night_colors = []

    def __init__(self):
        self.__config = configparser.ConfigParser()
        self.__read()

    def __read(self):
        path = os.path.dirname(os.path.abspath(__file__))
        self.__config.read(path + '/config.ini')
        self.night_mode_on = self.__config['MODE']['night_mode_on'] != '0'
        self.night_colors = self.__config['NIGHT_COLORS']
        self.day_colors = self.__config['DAY_COLORS']
        self.device = self.__config['CONNECTION']['device']

    def set(self, section, option, value):
        self.__config.set(section, option, value)
        self.__config['MODE']['night_mode_on'] = value

        with open("config.ini", 'w') as configfile:
            self.__config.write(configfile)

        self.__read()



import sys

from honda_dashboard.dashboard import Dashboard


def main(args=None):
    """The main routine."""
    if args is None:
        args = sys.argv[1:]

    app = Dashboard()
    app.run()


if __name__ == "__main__":
    main()



import qtawesome as qta
from PyQt5.QtWidgets import QToolButton, QToolBar, QAction

from honda_dashboard.config import Config


class Toolbar(QToolBar):

    def __init__(self, cfg: Config):
        super().__init__()

        self.__config = cfg
        self.toggle_night_mode = QAction()

        # Add night mode button.
        self.__night_mode_btn = _NightModeBtn(cfg)
        self.__night_mode_btn.clicked.connect(self.__night_mode_toggle)
        self.addWidget(self.__night_mode_btn)

        self.handle_mode()

    def handle_mode(self):
        if self.__config.night_mode_on:
            text_color = self.__config.night_colors['text']
            bg = self.__config.night_colors['bg']
        else:
            text_color = self.__config.day_colors['text']
            bg = self.__config.day_colors['bg']

        self.setStyleSheet('background-color: #' + bg + '; color: #' + text_color + ';')
        self.__night_mode_btn.handle_mode()

    def __night_mode_toggle(self):
        self.toggle_night_mode.trigger()


class _NightModeBtn(QToolButton):

    __icon_color = None

    def __init__(self, cfg: Config):
        super().__init__()

        self.__config = cfg

        self.setIcon(qta.icon('ei.adjust'))
        self.setToolTip('Toggle night/day mode.')
        self.setBaseSize(32, 32)
        self.isCheckable()

    def handle_mode(self):
        if self.__config.night_mode_on:
            text_color = self.__config.night_colors['text']
            bg = self.__config.night_colors['bg']
            hover = self.__config.night_colors['hover']
        else:
            text_color = self.__config.day_colors['text']
            bg = self.__config.day_colors['bg']
            hover = self.__config.day_colors['hover']

        self.setStyleSheet('_NightModeBtn {background-color: #' + bg + ';}'
                           + '_NightModeBth:hover {background-color: #' + hover + ';}')
        self.setIcon(qta.icon('ei.adjust', color='#' + text_color))





import string

from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import QWidget, QLabel, QVBoxLayout
from obd import OBDResponse

from honda_dashboard.config import Config


class Limits:

    def __init__(self, minimum: int, maximum: int, critical: int):
        self.min = minimum
        self.max = maximum
        self.critical = critical


class IndicatorBox(QWidget):

    def __init__(self, title: string, unit: string, cfg: Config, rounding=False, limits: Limits = None):
        super().__init__()

        self.__title = _Title(title, cfg)
        self.__value = _Value(unit, cfg, rounding, limits)
        self.__config = cfg

        self.setLayout(QVBoxLayout())
        self.layout().addWidget(self.__title)
        self.layout().addWidget(self.__value)

        self.handle_mode()

    def update_value(self, value: OBDResponse):
        self.__value.update_value(value)

    def handle_mode(self):
        self.__title.handle_mode()
        self.__value.handle_mode()


class _Value(QLabel):

    __color = None
    __value = None

    def __init__(self, unit: string, cfg: Config, rounding=False, limits: Limits = None):
        super().__init__()

        self.__config = cfg
        self.__unit = unit
        self.__rounding = rounding
        self.__limits = limits
        self.__limits_color_processor = _LimitsColorProcessor(limits, cfg)

        font = QFont('Arial', 20)
        font.setBold(True)
        self.setFont(font)

    def update_value(self, value: OBDResponse):
        if self.__should_update(value) is False:
            return

        self.__value = value.value
        val = self.__process_value(value)
        self.setText(val + ' ' + self.__unit)

        if self.__limits is None:
            return

        self.__update_color(val)

    def __should_update(self, value: OBDResponse) -> bool:
        if value.value != self.__value:
            self.__value = value.value

            return True

        return False

    def __update_color(self, val):
        color = self.__limits_color_processor.process(val)

        if self.__color != color:
            self.__color = color
            self.setStyleSheet('color: #' + color + ';')

    def handle_mode(self):
        if self.__config.night_mode_on:
            color = self.__config.night_colors['text']
        else:
            color = self.__config.day_colors['text']

        self.setStyleSheet('color: #' + color + ';')

    def __process_value(self, value: OBDResponse) -> string:
        val = str(value.value).split(' ')[0]

        if self.__rounding:
            val = round(float(val))

        return str(val)


class _Title(QLabel):

    def __init__(self, text: string, cfg: Config):
        super().__init__()

        self.__config = cfg
        self.setText(text)
        self.setFont(QFont('Arial', 14))

    def handle_mode(self):
        if self.__config.night_mode_on:
            color = self.__config.night_colors['text']
        else:
            color = self.__config.day_colors['text']

        self.setStyleSheet('color: #' + color + ';')


class _LimitsColorProcessor:
    __MIN_COLOR_KEY = 'min'
    __NORMAL_COLOR_KEY = 'normal'
    __MAX_COLOR_KEY = 'max'
    __CRITICAL_COLOR_KEY = 'critical'

    def __init__(self, limits: Limits, cfg: Config):
        self.__limits = limits
        self.__config = cfg

    def process(self, value: string) -> string:
        color_key = self.__color_key(value)

        if self.__config.night_mode_on:
            return self.__config.night_colors[color_key]

        return self.__config.day_colors[color_key]

    def __color_key(self, value: string) -> string:
        val = int(value)

        if val <= self.__limits.min:
            return self.__MIN_COLOR_KEY

        if self.__limits.min < val <= self.__limits.max:
            return self.__NORMAL_COLOR_KEY

        if self.__limits.max < val <= self.__limits.critical:
            return self.__MAX_COLOR_KEY

        return self.__CRITICAL_COLOR_KEY



from setuptools import setup

setup(
    name='honda-dashboard',
    version='0.1.3',
    description='OBD2 Honda honda_dashboard.',
    url='https://github.com/todstoychev/dashboard',
    author='Todor Todorov',
    author_email='todstoychev@gmail.com',
    license='BSD 2-clause',
    packages=['honda_dashboard'],
    install_requires=['qtawesome',
                      'pyqt5',
                      'obd'],

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    entry_points={
        'console_scripts': [
            'honda-dashboard=honda_dashboard.__main__:main',
        ],
    },
    package_data={
        '': ['config.ini', 'honda.png']
    },
)
